@@ -1,5 +1,6 @@
 /** @odoo-module **/
 
+import { DropdownItem } from "@web/core/dropdown/dropdown_item";
 import { registry } from "@web/core/registry";
 import { useService } from "@web/core/utils/hooks";
 
@@ -26,23 +27,23 @@ export class ImportRecords extends Component {
         this.action.doAction({
             type: "ir.actions.client",
             tag: "import",
-            params: { model: resModel, context }
+            params: { model: resModel, context },
         });
     }
 }
 
 ImportRecords.template = "base_import.ImportRecords";
+ImportRecords.components = { DropdownItem };
 
-const importRecordsItem = {
+export const importRecordsItem = {
     Component: ImportRecords,
     groupNumber: 4,
     isDisplayed: ({ config, isSmall }) =>
         !isSmall &&
         config.actionType === "ir.actions.act_window" &&
-        ["kanban", "list"].includes(config.viewType)
-        // TODO: add arch info to searchModel?
-        // !!JSON.parse(env.view.arch.attrs.import || "1") &&
-        // !!JSON.parse(env.view.arch.attrs.create || "1"),
+        ["kanban", "list"].includes(config.viewType) &&
+        !!JSON.parse(config.viewArch.getAttribute("import") || "1") &&
+        !!JSON.parse(config.viewArch.getAttribute("create") || "1"),
 };
 
 favoriteMenuRegistry.add("import-menu", importRecordsItem, { sequence: 1 });
