@@ -4,9 +4,9 @@
 {
     'name': 'Denmark - Accounting',
     'version': '1.0',
-    'author': 'Odoo House ApS',
-    'website': 'https://odoohouse.dk',
-    'category': 'Localization',
+    'author': 'Odoo House ApS, VK DATA ApS',
+    'website': 'http://odoodanmark.dk',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': """
 
 Localization Module for Denmark
@@ -22,7 +22,7 @@ This is the module to manage the **accounting chart for Denmark**. Cover both on
         - 25% moms
         - Resturationsmoms 6,25%
         - Omvendt betalingspligt
-        
+
 - Konteringsgrupper
         - EU (Virksomhed)
         - EU (Privat)
@@ -87,7 +87,6 @@ Produkt setup:
 
 .
 
-Copyright 2018 Odoo House ApS
     """,
     'depends': ['account', 'base_iban', 'base_vat'],
     'data': [
@@ -95,12 +94,15 @@ Copyright 2018 Odoo House ApS
         'data/l10n_dk_chart_template_data.xml',
         'data/account.account.template.csv',
         'data/l10n_dk_chart_template_post_data.xml',
+        'data/account_tax_report_data.xml',
         'data/account_tax_template_data.xml',
         'data/account_fiscal_position_template.xml',
         'data/account_fiscal_position_tax_template.xml',
         'data/account_fiscal_position_account_template.xml',
         'data/account_chart_template_configuration_data.xml',
-        'data/menuitem_data.xml'
+    ],
+    'demo': [
+        'demo/demo_company.xml',
     ],
     'license': 'LGPL-3',
 }
