@@ -2,7 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, fields, models, _
-from odoo.exceptions import AccessError, UserError
+from odoo.exceptions import UserError, ValidationError
 from odoo.tools import float_compare, float_round
 from odoo.osv import expression
 
@@ -29,6 +29,7 @@ class MrpUnbuild(models.Model):
         required=True, states={'done': [('readonly', True)]})
     product_uom_id = fields.Many2one(
         'uom.uom', 'Unit of Measure',
+        compute='_compute_product_uom_id', store=True, readonly=False, precompute=True,
         required=True, states={'done': [('readonly', True)]})
     bom_id = fields.Many2one(
         'mrp.bom', 'Bill of Material',
@@ -51,19 +52,20 @@ class MrpUnbuild(models.Model):
         states={'done': [('readonly', True)]}, check_company=True)
     mo_bom_id = fields.Many2one('mrp.bom', 'Bill of Material used on the Production Order', related='mo_id.bom_id')
     lot_id = fields.Many2one(
-        'stock.production.lot', 'Lot/Serial Number',
-        domain="[('product_id', '=', product_id), ('company_id', '=', company_id)]", check_company=True,
-        states={'done': [('readonly', True)]}, help="Lot/Serial Number of the product to unbuild.")
+        'stock.lot', 'Lot/Serial Number',
+        domain="[('product_id', '=', product_id), ('company_id', '=', company_id)]", check_company=True)
     has_tracking=fields.Selection(related='product_id.tracking', readonly=True)
     location_id = fields.Many2one(
         'stock.location', 'Source Location',
         domain="[('usage','=','internal'), '|', ('company_id', '=', False), ('company_id', '=', company_id)]",
         check_company=True,
+        compute='_compute_location_id', store=True, readonly=False, precompute=True,
         required=True, states={'done': [('readonly', True)]}, help="Location where the product you want to unbuild is.")
     location_dest_id = fields.Many2one(
         'stock.location', 'Destination Location',
         domain="[('usage','=','internal'), '|', ('company_id', '=', False), ('company_id', '=', company_id)]",
         check_company=True,
+        compute='_compute_location_id', store=True, readonly=False, precompute=True,
         required=True, states={'done': [('readonly', True)]}, help="Location where you want to send the components resulting from the unbuild order.")
     consume_line_ids = fields.One2many(
         'stock.move', 'consume_unbuild_id', readonly=True,
@@ -75,17 +77,23 @@ class MrpUnbuild(models.Model):
         ('draft', 'Draft'),
         ('done', 'Done')], string='Status', default='draft')
 
-    @api.onchange('company_id')
-    def _onchange_company_id(self):
-        if self.company_id:
-            warehouse = self.env['stock.warehouse'].search([('company_id', '=', self.company_id.id)], limit=1)
-            if self.location_id.company_id != self.company_id:
-                self.location_id = warehouse.lot_stock_id
-            if self.location_dest_id.company_id != self.company_id:
-                self.location_dest_id = warehouse.lot_stock_id
-        else:
-            self.location_id = False
-            self.location_dest_id = False
+    @api.depends('mo_id', 'product_id')
+    def _compute_product_uom_id(self):
+        for record in self:
+            if record.mo_id.product_id and record.mo_id.product_id == record.product_id:
+                record.product_uom_id = record.mo_id.product_uom_id.id
+            else:
+                record.product_uom_id = record.product_id.uom_id.id
+
+    @api.depends('company_id')
+    def _compute_location_id(self):
+        for order in self:
+            if order.company_id:
+                warehouse = self.env['stock.warehouse'].search([('company_id', '=', order.company_id.id)], limit=1)
+                if order.location_id.company_id != order.company_id:
+                    order.location_id = warehouse.lot_stock_id
+                if order.location_dest_id.company_id != order.company_id:
+                    order.location_dest_id = warehouse.lot_stock_id
 
     @api.onchange('mo_id')
     def _onchange_mo_id(self):
@@ -93,11 +101,13 @@ class MrpUnbuild(models.Model):
             self.product_id = self.mo_id.product_id.id
             self.bom_id = self.mo_id.bom_id
             self.product_uom_id = self.mo_id.product_uom_id
+            self.lot_id = self.mo_id.lot_producing_id
             if self.has_tracking == 'serial':
                 self.product_qty = 1
             else:
                 self.product_qty = self.mo_id.product_qty
 
+
     @api.onchange('product_id')
     def _onchange_product_id(self):
         if self.product_id:
@@ -108,13 +118,14 @@ class MrpUnbuild(models.Model):
     def _check_qty(self):
         for unbuild in self:
             if unbuild.product_qty <= 0:
-                raise ValueError(_('Unbuild Order product quantity has to be strictly positive.'))
+                raise ValidationError(_('Unbuild Order product quantity has to be strictly positive.'))
 
-    @api.model
-    def create(self, vals):
-        if not vals.get('name') or vals['name'] == _('New'):
-            vals['name'] = self.env['ir.sequence'].next_by_code('mrp.unbuild') or _('New')
-        return super(MrpUnbuild, self).create(vals)
+    @api.model_create_multi
+    def create(self, vals_list):
+        for vals in vals_list:
+            if not vals.get('name') or vals['name'] == _('New'):
+                vals['name'] = self.env['ir.sequence'].next_by_code('mrp.unbuild') or _('New')
+        return super().create(vals_list)
 
     @api.ondelete(at_uninstall=False)
     def _unlink_except_done(self):
@@ -134,7 +145,7 @@ class MrpUnbuild(models.Model):
         consume_moves = self._generate_consume_moves()
         consume_moves._action_confirm()
         produce_moves = self._generate_produce_moves()
-        produce_moves._action_confirm()
+        produce_moves.with_context(default_lot_id=False)._action_confirm()
 
         finished_moves = consume_moves.filtered(lambda m: m.product_id == self.product_id)
         consume_moves -= finished_moves
@@ -194,7 +205,11 @@ class MrpUnbuild(models.Model):
         consume_moves.mapped('move_line_ids').write({'produce_line_ids': [(6, 0, produced_move_line_ids.ids)]})
         if self.mo_id:
             unbuild_msg = _(
-                "%s %s unbuilt in", self.product_qty, self.product_uom_id.name) + " <a href=# data-oe-model=mrp.unbuild data-oe-id=%d>%s</a>" % (self.id, self.display_name)
+                "%(qty)s %(measure)s unbuilt in %(order)s",
+                qty=self.product_qty,
+                measure=self.product_uom_id.name,
+                order=self._get_html_link(),
+            )
             self.mo_id.message_post(
                 body=unbuild_msg,
                 subtype_id=self.env.ref('mail.mt_note').id)
@@ -207,7 +222,7 @@ class MrpUnbuild(models.Model):
                 finished_moves = unbuild.mo_id.move_finished_ids.filtered(lambda move: move.state == 'done')
                 factor = unbuild.product_qty / unbuild.mo_id.product_uom_id._compute_quantity(unbuild.mo_id.product_qty, unbuild.product_uom_id)
                 for finished_move in finished_moves:
-                    moves += unbuild._generate_move_from_existing_move(finished_move, factor, finished_move.location_dest_id, finished_move.location_id)
+                    moves += unbuild._generate_move_from_existing_move(finished_move, factor, unbuild.location_id, finished_move.location_id)
             else:
                 factor = unbuild.product_uom_id._compute_quantity(unbuild.product_qty, unbuild.bom_id.product_uom_id) / unbuild.bom_id.product_qty
                 moves += unbuild._generate_move_from_bom_line(self.product_id, self.product_uom_id, unbuild.product_qty)
