@@ -11,20 +11,30 @@ class ReceptionReport(models.AbstractModel):
     _name = 'report.stock.report_reception'
     _description = "Stock Reception Report"
 
+    @api.model
+    def get_report_data(self, docids, data):
+        report_values = self._get_report_values(docids, data)
+        report_values['docs'] = self._format_html_docs(report_values.get('docs', False))
+        report_values['sources_info'] = self._format_html_sources_info(report_values.get('sources_to_lines', {}))
+        report_values['sources_to_lines'] = self._format_html_sources_to_lines(report_values.get('sources_to_lines', {}))
+        report_values['sources_to_formatted_scheduled_date'] = self._format_html_sources_to_date(report_values.get('sources_to_formatted_scheduled_date', {}))
+        report_values['show_uom'] = self.env.user.has_group('uom.group_uom')
+        return report_values
+
     @api.model
     def _get_report_values(self, docids, data=None):
         ''' This report is flexibly designed to work with both individual and batch pickings.
         '''
-        docids = self.env.context.get('default_picking_ids', docids)
-        pickings = self.env['stock.picking'].search([('id', 'in', docids), ('picking_type_code', '!=', 'outgoing'), ('state', '!=', 'cancel')])
-        picking_states = pickings.mapped('state')
+        docs = self._get_docs(docids)
+        doc_states = docs.mapped('state')
         # unsupported cases
-        if not pickings:
-            msg = _("No transfers selected or a delivery order selected")
-        elif 'done' in picking_states and len(set(picking_states)) > 1:
-            pickings = False
-            msg = _("This report cannot be used for done and not done transfers at the same time")
-        if not pickings:
+        doc_types = self._get_doc_types()
+        if not docs:
+            msg = _("No %s selected or a delivery order selected", doc_types)
+        elif 'done' in doc_states and len(set(doc_states)) > 1:
+            docs = False
+            msg = _("This report cannot be used for done and not done %s at the same time", doc_types)
+        if not docs:
             return {'pickings': False, 'reason': msg}
 
         # incoming move qtys
@@ -33,13 +43,13 @@ class ReceptionReport(models.AbstractModel):
         product_to_total_assigned = defaultdict(lambda: [0.0, []])
 
         # to support batch pickings we need to track the total already assigned
-        move_lines = pickings.move_lines.filtered(lambda m: m.product_id.type == 'product' and m.state != 'cancel')
-        assigned_moves = move_lines.mapped('move_dest_ids')
+        move_ids = self._get_moves(docs)
+        assigned_moves = move_ids.mapped('move_dest_ids')
         product_to_assigned_qty = defaultdict(float)
         for assigned in assigned_moves:
             product_to_assigned_qty[assigned.product_id] += assigned.product_qty
 
-        for move in move_lines:
+        for move in move_ids:
             qty_already_assigned = 0
             if move.move_dest_ids:
                 qty_already_assigned = min(product_to_assigned_qty[move.product_id], move.product_qty)
@@ -58,11 +68,11 @@ class ReceptionReport(models.AbstractModel):
                     product_to_qty_to_assign[move.product_id].append((quantity_to_assign - qty_already_assigned, move))
 
         # only match for non-mto moves in same warehouse
-        warehouse = pickings[0].picking_type_id.warehouse_id
+        warehouse = docs[0].picking_type_id.warehouse_id
         wh_location_ids = self.env['stock.location']._search([('id', 'child_of', warehouse.view_location_id.id), ('usage', '!=', 'supplier')])
 
         allowed_states = ['confirmed', 'partially_available', 'waiting']
-        if 'done' in picking_states:
+        if 'done' in doc_states:
             # only done moves are allowed to be assigned to already reserved moves
             allowed_states += ['assigned']
 
@@ -72,10 +82,9 @@ class ReceptionReport(models.AbstractModel):
                 ('product_qty', '>', 0),
                 ('location_id', 'in', wh_location_ids),
                 ('move_orig_ids', '=', False),
-                ('picking_id', 'not in', pickings.ids),
                 ('product_id', 'in',
                     [p.id for p in list(product_to_qty_to_assign.keys()) + list(product_to_qty_draft.keys())]),
-            ],
+            ] + self._get_extra_domain(docs),
             order='reservation_date, priority desc, date, id')
 
         products_to_outs = defaultdict(list)
@@ -95,7 +104,7 @@ class ReceptionReport(models.AbstractModel):
 
                 qty_to_reserve = out.product_qty
                 product_uom = out.product_id.uom_id
-                if 'done' not in picking_states and out.state == 'partially_available':
+                if 'done' not in doc_states and out.state == 'partially_available':
                     qty_to_reserve -= out.product_uom._compute_quantity(out.reserved_availability, product_uom)
                 moves_in_ids = []
                 qty_done = 0
@@ -154,10 +163,10 @@ class ReceptionReport(models.AbstractModel):
         return {
             'data': data,
             'doc_ids': docids,
-            'doc_model': 'stock.picking',
+            'doc_model': self._get_doc_model(),
             'sources_to_lines': sources_to_lines,
             'precision': self.env['decimal.precision'].precision_get('Product Unit of Measure'),
-            'pickings': pickings,
+            'docs': docs,
             'sources_to_formatted_scheduled_date': sources_to_formatted_scheduled_date,
         }
 
@@ -176,6 +185,22 @@ class ReceptionReport(models.AbstractModel):
             'move_ins': move_ins and move_ins.ids or False,
         }
 
+    def _get_docs(self, docids):
+        docids = self.env.context.get('default_picking_ids', docids)
+        return self.env['stock.picking'].search([('id', 'in', docids), ('picking_type_code', '!=', 'outgoing'), ('state', '!=', 'cancel')])
+
+    def _get_doc_model(self):
+        return 'stock.picking'
+
+    def _get_doc_types(self):
+        return "transfers"
+
+    def _get_moves(self, docs):
+        return docs.move_ids.filtered(lambda m: m.product_id.type == 'product' and m.state != 'cancel')
+
+    def _get_extra_domain(self, docs):
+        return [('picking_id', 'not in', docs.ids)]
+
     def _get_formatted_scheduled_date(self, source):
         """ Unfortunately different source record types have different field names for their "Scheduled Date"
         Therefore an extendable method is needed.
@@ -219,13 +244,13 @@ class ReceptionReport(models.AbstractModel):
                     out.move_line_ids.move_id = new_out
                     assigned_amount = 0
                     for move_line_id in new_out.move_line_ids:
-                        if assigned_amount + move_line_id.product_qty > qty_to_link:
-                            new_move_line = move_line_id.copy({'product_uom_qty': 0, 'qty_done': 0})
-                            new_move_line.product_uom_qty = move_line_id.product_uom_qty
-                            move_line_id.product_uom_qty = out.product_id.uom_id._compute_quantity(qty_to_link - assigned_amount, out.product_uom, rounding_method='HALF-UP')
-                            new_move_line.product_uom_qty -= out.product_id.uom_id._compute_quantity(move_line_id.product_qty, out.product_uom, rounding_method='HALF-UP')
+                        if assigned_amount + move_line_id.reserved_qty > qty_to_link:
+                            new_move_line = move_line_id.copy({'reserved_uom_qty': 0, 'qty_done': 0})
+                            new_move_line.reserved_uom_qty = move_line_id.reserved_uom_qty
+                            move_line_id.reserved_uom_qty = out.product_id.uom_id._compute_quantity(qty_to_link - assigned_amount, out.product_uom, rounding_method='HALF-UP')
+                            new_move_line.reserved_uom_qty -= out.product_id.uom_id._compute_quantity(move_line_id.reserved_qty, out.product_uom, rounding_method='HALF-UP')
                         move_line_id.move_id = out
-                        assigned_amount += move_line_id.product_qty
+                        assigned_amount += move_line_id.reserved_qty
                         if float_compare(assigned_amount, qty_to_link, precision_rounding=out.product_id.uom_id.rounding) == 0:
                             break
 
@@ -238,6 +263,7 @@ class ReceptionReport(models.AbstractModel):
 
                 linked_qty = min(in_move.product_qty, qty_to_link)
                 in_move.move_dest_ids |= out
+                self._action_assign(in_move, out)
                 out.procure_method = 'make_to_order'
                 quantity_remaining -= linked_qty
                 qty_to_link -= linked_qty
@@ -263,6 +289,7 @@ class ReceptionReport(models.AbstractModel):
             if out.id not in in_move.move_dest_ids.ids:
                 continue
             in_move.move_dest_ids -= out
+            self._action_unassign(in_move, out)
             amount_unassigned += min(qty, in_move.product_qty)
             if float_compare(qty, amount_unassigned, precision_rounding=out.product_id.uom_id.rounding) <= 0:
                 break
@@ -285,18 +312,65 @@ class ReceptionReport(models.AbstractModel):
                     for move_line_id in new_out.move_line_ids:
                         if reserved_amount_to_remain <= 0:
                             break
-                        if move_line_id.product_qty > reserved_amount_to_remain:
-                            new_move_line = move_line_id.copy({'product_uom_qty': 0, 'qty_done': 0})
-                            new_move_line.product_uom_qty = out.product_id.uom_id._compute_quantity(move_line_id.product_qty - reserved_amount_to_remain, move_line_id.product_uom_id, rounding_method='HALF-UP')
-                            move_line_id.product_uom_qty -= new_move_line.product_uom_qty
+                        if move_line_id.reserved_qty > reserved_amount_to_remain:
+                            new_move_line = move_line_id.copy({'reserved_uom_qty': 0, 'qty_done': 0})
+                            new_move_line.reserved_uom_qty = out.product_id.uom_id._compute_quantity(move_line_id.reserved_qty - reserved_amount_to_remain, move_line_id.product_uom_id, rounding_method='HALF-UP')
+                            move_line_id.reserved_uom_qty -= new_move_line.reserved_uom_qty
                             new_move_line.move_id = out
                             break
                         else:
                             move_line_id.move_id = out
-                            reserved_amount_to_remain -= move_line_id.product_qty
+                            reserved_amount_to_remain -= move_line_id.reserved_qty
                     (out | new_out)._compute_reserved_availability()
                 out.move_orig_ids = False
                 new_out._recompute_state()
         out.procure_method = 'make_to_stock'
         out._recompute_state()
         return True
+
+    def _action_assign(self, in_move, out_move):
+        """ For extension purposes only """
+        return
+
+    def _action_unassign(self, in_move, out_move):
+        """ For extension purposes only """
+        return
+
+    def _format_html_docs(self, docs):
+        """ Format docs to be sent in an html request. """
+        return [{
+            'id': doc.id,
+            'name': doc.display_name,
+            'state': doc.state,
+            'display_state': dict(doc._fields['state']._description_selection(self.env)).get(doc.state),
+        } for doc in docs] if docs else docs
+
+    def _format_html_sources_to_date(self, sources_to_dates):
+        """ Format sources_to_formatted_scheduled_date to be sent in an html request. """
+        return {str(source): date for (source, date) in sources_to_dates.items()}
+
+    def _format_html_sources_to_lines(self, sources_to_lines):
+        """ Format sources_to_lines to be sent in an html request, while adding an index for OWL's t-foreach. """
+        return {
+            str(source): [{**line, 'index': i, 'move_out_id': line['move_out'].id} for i, line in enumerate(lines)]
+            for source, lines in sources_to_lines.items()
+        }
+
+    def _format_html_sources_info(self, sources_to_lines):
+        """ Format used info from sources of sources_to_lines to be sent in an html request. """
+        return {str(source): [self._format_html_source(s, s._name == 'stock.picking')for s in source] for source in sources_to_lines.keys()}
+
+    def _format_html_source(self, source, is_picking=False):
+        """ Format used info from a single source to be sent in an html request. """
+        formatted = {
+            'id': source.id,
+            'model': source._name,
+            'name': source.display_name,
+        }
+        if is_picking:
+            formatted.update({
+                'priority': source.priority,
+                'partner_id': source.partner_id.id if source.partner_id else False,
+                'partner_name': source.partner_id.name if source.partner_id else False,
+            })
+        return formatted
