@@ -3,10 +3,8 @@
 
 from . import account_chart_template
 from . import account_move
-from . import account_reconciliation_widget
 from . import product
 from . import stock_move
-from . import stock_inventory
 from . import stock_location
 from . import stock_move_line
 from . import stock_picking
