@@ -83,11 +83,12 @@ class StockLandedCost(models.Model):
         if self.target_model != 'picking':
             self.picking_ids = False
 
-    @api.model
-    def create(self, vals):
-        if vals.get('name', _('New')) == _('New'):
-            vals['name'] = self.env['ir.sequence'].next_by_code('stock.landed.cost')
-        return super().create(vals)
+    @api.model_create_multi
+    def create(self, vals_list):
+        for vals in vals_list:
+            if vals.get('name', _('New')) == _('New'):
+                vals['name'] = self.env['ir.sequence'].next_by_code('stock.landed.cost')
+        return super().create(vals_list)
 
     def unlink(self):
         self.button_cancel()
@@ -176,7 +177,11 @@ class StockLandedCost(models.Model):
             cost.write(cost_vals)
             if cost.account_move_id:
                 move._post()
+            cost.reconcile_landed_cost()
+        return True
 
+    def reconcile_landed_cost(self):
+        for cost in self:
             if cost.vendor_bill_id and cost.vendor_bill_id.state == 'posted' and cost.company_id.anglo_saxon_accounting:
                 all_amls = cost.vendor_bill_id.line_ids | cost.account_move_id.line_ids
                 for product in cost.cost_lines.product_id:
@@ -184,8 +189,6 @@ class StockLandedCost(models.Model):
                     input_account = accounts['stock_input']
                     all_amls.filtered(lambda aml: aml.account_id == input_account and not aml.reconciled).reconcile()
 
-        return True
-
     def get_valuation_lines(self):
         self.ensure_one()
         lines = []
@@ -279,7 +282,7 @@ class StockLandedCost(models.Model):
         return dict(action, domain=domain)
 
     def _get_targeted_move_ids(self):
-        return self.picking_ids.move_lines
+        return self.picking_ids.move_ids
 
     def _check_can_validate(self):
         if any(cost.state != 'draft' for cost in self):
