@@ -5,6 +5,6 @@ class ThemeCobalt(models.AbstractModel):
     _inherit = 'theme.utils'
 
     def _theme_cobalt_post_copy(self, mod):
-        self.enable_asset('Ripple effect SCSS')
-        self.enable_asset('Ripple effect JS')
+        self.enable_asset("website.ripple_effect_scss")
+        self.enable_asset("website.ripple_effect_js")
         self.enable_header_off_canvas()
