@@ -1,19 +1,25 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import models, fields, api
-from odoo.tools.pdf import OdooPdfFileReader, OdooPdfFileWriter
+from odoo import models, fields, api, _
+from odoo.tools.pdf import OdooPdfFileReader
 from odoo.osv import expression
 from odoo.tools import html_escape
 from odoo.exceptions import RedirectWarning
+try:
+    from PyPDF2.errors import PdfReadError
+except ImportError:
+    from PyPDF2.utils import PdfReadError
 
 from lxml import etree
+from struct import error as StructError
 import base64
 import io
 import logging
 import pathlib
 import re
 
+
 _logger = logging.getLogger(__name__)
 
 
@@ -28,7 +34,6 @@ class AccountEdiFormat(models.Model):
         ('unique_code', 'unique (code)', 'This code already exists')
     ]
 
-
     ####################################################
     # Low-level methods
     ####################################################
@@ -38,8 +43,14 @@ class AccountEdiFormat(models.Model):
         edi_formats = super().create(vals_list)
 
         # activate by default on journal
-        journals = self.env['account.journal'].search([])
-        journals._compute_edi_format_ids()
+        if not self.pool.loaded:
+            # The registry is not totally loaded. We cannot yet recompute the field on jourals as
+            # The helper methods aren't yet overwritten by all installed `l10n_` modules.
+            # Delay it in the register hook
+            self.pool._delay_compute_edi_format_ids = True
+        else:
+            journals = self.env['account.journal'].search([])
+            journals._compute_edi_format_ids()
 
         # activate cron
         if any(edi_format._needs_web_services() for edi_format in edi_formats):
@@ -47,41 +58,30 @@ class AccountEdiFormat(models.Model):
 
         return edi_formats
 
+    def _register_hook(self):
+        if hasattr(self.pool, "_delay_compute_edi_format_ids"):
+            del self.pool._delay_compute_edi_format_ids
+            journals = self.env['account.journal'].search([])
+            journals._compute_edi_format_ids()
+
+        return super()._register_hook()
+
     ####################################################
     # Export method to override based on EDI Format
     ####################################################
 
-    def _get_invoice_edi_content(self, move):
-        ''' Create a bytes literal of the file content representing the invoice - to be overridden by the EDI Format
-        :returns:       bytes literal of the content generated (typically XML).
-        '''
-        return b''
+    def _get_move_applicability(self, move):
+        """ Core function for the EDI processing: it first checks whether the EDI format is applicable on a given
+        move, if so, it then returns a dictionary containing the functions to call for this move.
 
-    def _get_payment_edi_content(self, move):
-        ''' Create a bytes literal of the file content representing the payment - to be overridden by the EDI Format
-        :returns:       bytes literal of the content generated (typically XML).
-        '''
-        return b''
-
-    def _is_required_for_invoice(self, invoice):
-        """ Indicate if this EDI must be generated for the invoice passed as parameter.
-
-        :param invoice: An account.move having the invoice type.
-        :returns:       True if the EDI must be generated, False otherwise.
+        :return: dict mapping str to function (callable)
+        * post:             function called for edi.documents with state 'to_send' (post flow)
+        * cancel:           function called for edi.documents with state 'to_cancel' (cancel flow)
+        * post_batching:    function returning the batching key for the post flow
+        * cancel_batching:  function returning the batching key for the cancel flow
+        * edi_content:      function called when computing the edi_content for an edi.document
         """
-        # TO OVERRIDE
         self.ensure_one()
-        return True
-
-    def _is_required_for_payment(self, payment):
-        """ Indicate if this EDI must be generated for the payment passed as parameter.
-
-        :param payment: An account.move linked to either an account.payment, either an account.bank.statement.line.
-        :returns:       True if the EDI must be generated, False otherwise.
-        """
-        # TO OVERRIDE
-        self.ensure_one()
-        return False
 
     def _needs_web_services(self):
         """ Indicate if the EDI must be generated asynchronously through to some web services.
@@ -111,56 +111,6 @@ class AccountEdiFormat(models.Model):
         """
         return True
 
-    def _is_embedding_to_invoice_pdf_needed(self):
-        """ Indicate if the EDI must be embedded inside the PDF report.
-
-        :returns: True if the documents need to be embedded, False otherwise.
-        """
-        # TO OVERRIDE
-        return False
-
-    def _get_embedding_to_invoice_pdf_values(self, invoice):
-        """ Get the values to embed to pdf.
-
-        :returns:   A dictionary {'name': name, 'datas': datas} or False if there are no values to embed.
-        * name:     The name of the file.
-        * datas:    The bytes ot the file.
-        To remove in master
-        """
-        self.ensure_one()
-        attachment = invoice._get_edi_attachment(self)
-        if not attachment or not self._is_embedding_to_invoice_pdf_needed():
-            return False
-        datas = base64.b64decode(attachment.with_context(bin_size=False).datas)
-        return {'name': attachment.name, 'datas': datas}
-
-    def _support_batching(self, move, state, company):
-        """ Indicate if we can send multiple documents in the same time to the web services.
-        If True, the _post_%s_edi methods will get multiple documents in the same time.
-        Otherwise, these methods will be called with only one record at a time.
-
-        :param move:    The move that we are trying to batch.
-        :param state:   The EDI state of the move.
-        :param company: The company with which we are sending the EDI.
-        :returns:       True if batching is supported, False otherwise.
-        """
-        # TO OVERRIDE
-        return False
-
-    def _get_batch_key(self, move, state):
-        """ Returns a tuple that will be used as key to partitionnate the invoices/payments when creating batches
-        with multiple invoices/payments.
-        The type of move (invoice or payment), its company_id, its edi state and the edi_format are used by default, if
-        no further partition is needed for this format, this method should return (). It's not necessary to repeat those
-        fields in the custom key.
-
-        :param move:    The move to batch.
-        :param state:   The EDI state of the move.
-        :returns: The key to be used when partitionning the batches.
-        """
-        move.ensure_one()
-        return ()
-
     def _check_move_configuration(self, move):
         """ Checks the move and relevant records for potential error (missing data, etc).
 
@@ -170,68 +120,16 @@ class AccountEdiFormat(models.Model):
         # TO OVERRIDE
         return []
 
-    def _post_invoice_edi(self, invoices):
-        """ Create the file content representing the invoice (and calls web services if necessary).
-
-        :param invoices:    A list of invoices to post.
-        :returns:           A dictionary with the invoice as key and as value, another dictionary:
-        * success:          True if the edi was successfully posted.
-        * attachment:       The attachment representing the invoice in this edi_format.
-        * error:            An error if the edi was not successfully posted.
-        * blocking_level:   (optional) How bad is the error (how should the edi flow be blocked ?)
-        """
-        # TO OVERRIDE
-        self.ensure_one()
-        return {}
-
-    def _cancel_invoice_edi(self, invoices):
-        """Calls the web services to cancel the invoice of this document.
-
-        :param invoices:    A list of invoices to cancel.
-        :returns:           A dictionary with the invoice as key and as value, another dictionary:
-        * success:          True if the invoice was successfully cancelled.
-        * error:            An error if the edi was not successfully cancelled.
-        * blocking_level:   (optional) How bad is the error (how should the edi flow be blocked ?)
-        """
-        # TO OVERRIDE
-        self.ensure_one()
-        return {invoice: {'success': True} for invoice in invoices}  # By default, cancel succeeds doing nothing.
-
-    def _post_payment_edi(self, payments):
-        """ Create the file content representing the payment (and calls web services if necessary).
-
-        :param payments:   The payments to post.
-        :returns:           A dictionary with the payment as key and as value, another dictionary:
-        * attachment:       The attachment representing the payment in this edi_format if the edi was successfully posted.
-        * error:            An error if the edi was not successfully posted.
-        * blocking_level:   (optional) How bad is the error (how should the edi flow be blocked ?)
-        """
-        # TO OVERRIDE
-        self.ensure_one()
-        return {}
-
-    def _cancel_payment_edi(self, payments):
-        """Calls the web services to cancel the payment of this document.
-
-        :param payments:  A list of payments to cancel.
-        :returns:         A dictionary with the payment as key and as value, another dictionary:
-        * success:        True if the payment was successfully cancelled.
-        * error:          An error if the edi was not successfully cancelled.
-        * blocking_level: (optional) How bad is the error (how should the edi flow be blocked ?)
-        """
-        # TO OVERRIDE
-        self.ensure_one()
-        return {payment: {'success': True} for payment in payments}  # By default, cancel succeeds doing nothing.
-
     ####################################################
     # Import methods to override based on EDI Format
     ####################################################
 
-    def _create_invoice_from_xml_tree(self, filename, tree):
+    def _create_invoice_from_xml_tree(self, filename, tree, journal=None):
         """ Create a new invoice with the data inside the xml.
 
         :param filename: The name of the xml.
         :param tree:     The tree of the xml to import.
+        :param journal:  The journal on which importing the invoice.
         :returns:        The created invoice.
         """
         # TO OVERRIDE
@@ -307,35 +205,6 @@ class AccountEdiFormat(models.Model):
         """
         # TO OVERRIDE
         self.ensure_one()
-        if self._is_embedding_to_invoice_pdf_needed() and edi_document.attachment_id:
-            pdf_writer.embed_odoo_attachment(edi_document.attachment_id)
-
-    ####################################################
-    # Export Internal methods (not meant to be overridden)
-    ####################################################
-
-    def _embed_edis_to_pdf(self, pdf_content, invoice):
-        """ Create the EDI document of the invoice and embed it in the pdf_content.
-
-        :param pdf_content: the bytes representing the pdf to add the EDIs to.
-        :param invoice: the invoice to generate the EDI from.
-        :returns: the same pdf_content with the EDI of the invoice embed in it.
-        """
-        to_embed = invoice.edi_document_ids
-        # Add the attachments to the pdf file
-        if to_embed:
-            reader_buffer = io.BytesIO(pdf_content)
-            reader = OdooPdfFileReader(reader_buffer, strict=False)
-            writer = OdooPdfFileWriter()
-            writer.cloneReaderDocumentRoot(reader)
-            for edi_document in to_embed:
-                edi_document.edi_format_id._prepare_invoice_report(writer, edi_document)
-            buffer = io.BytesIO()
-            writer.write(buffer)
-            pdf_content = buffer.getvalue()
-            reader_buffer.close()
-            buffer.close()
-        return pdf_content
 
     ####################################################
     # Import Internal methods (not meant to be overridden)
@@ -385,14 +254,14 @@ class AccountEdiFormat(models.Model):
             pdf_reader = OdooPdfFileReader(buffer, strict=False)
         except Exception as e:
             # Malformed pdf
-            _logger.exception("Error when reading the pdf: %s" % e)
+            _logger.warning("Error when reading the pdf: %s", e, exc_info=True)
             return to_process
 
         # Process embedded files.
         try:
             for xml_name, content in pdf_reader.getAttachments():
                 to_process.extend(self._decode_xml(xml_name, content))
-        except NotImplementedError as e:
+        except (NotImplementedError, StructError, PdfReadError) as e:
             _logger.warning("Unable to access the attachments of %s. Tried to decrypt it, but %s." % (filename, e))
 
         # Process the pdf itself.
@@ -437,16 +306,19 @@ class AccountEdiFormat(models.Model):
         content = base64.b64decode(attachment.with_context(bin_size=False).datas)
         to_process = []
 
+        # XML attachments received by mail have a 'text/plain' mimetype (cfr. context key: 'attachments_mime_plainxml')
+        # Therefore, if content start with '<?xml', or if the filename ends with '.xml', it is considered as XML.
+        is_text_plain_xml = 'text/plain' in attachment.mimetype and (content.startswith(b'<?xml') or attachment.name.endswith('.xml'))
         if 'pdf' in attachment.mimetype:
             to_process.extend(self._decode_pdf(attachment.name, content))
-        elif 'xml' in attachment.mimetype:
+        elif attachment.mimetype.endswith('/xml') or is_text_plain_xml:
             to_process.extend(self._decode_xml(attachment.name, content))
         else:
             to_process.extend(self._decode_binary(attachment.name, content))
 
         return to_process
 
-    def _create_invoice_from_attachment(self, attachment):
+    def _create_document_from_attachment(self, attachment):
         """Decodes an ir.attachment to create an invoice.
 
         :param attachment:  An ir.attachment record.
@@ -466,9 +338,13 @@ class AccountEdiFormat(models.Model):
                 except RedirectWarning as rw:
                     raise rw
                 except Exception as e:
-                    _logger.exception("Error importing attachment \"%s\" as invoice with format \"%s\"", file_data['filename'], edi_format.name, str(e))
+                    _logger.exception(
+                        "Error importing attachment \"%s\" as invoice with format \"%s\": %s",
+                        file_data['filename'],
+                        edi_format.name,
+                        str(e))
                 if res:
-                    return res
+                    return res._link_invoice_origin_to_purchase_orders(timeout=4)
         return self.env['account.move']
 
     def _update_invoice_from_attachment(self, attachment, invoice):
@@ -482,16 +358,20 @@ class AccountEdiFormat(models.Model):
                 res = False
                 try:
                     if file_data['type'] == 'xml':
-                        res = edi_format.with_company(self.env.company)._update_invoice_from_xml_tree(file_data['filename'], file_data['xml_tree'], invoice)
+                        res = edi_format.with_company(invoice.company_id)._update_invoice_from_xml_tree(file_data['filename'], file_data['xml_tree'], invoice)
                     elif file_data['type'] == 'pdf':
-                        res = edi_format.with_company(self.env.company)._update_invoice_from_pdf_reader(file_data['filename'], file_data['pdf_reader'], invoice)
+                        res = edi_format.with_company(invoice.company_id)._update_invoice_from_pdf_reader(file_data['filename'], file_data['pdf_reader'], invoice)
                         file_data['pdf_reader'].stream.close()
                     else:  # file_data['type'] == 'binary'
                         res = edi_format._update_invoice_from_binary(file_data['filename'], file_data['content'], file_data['extension'], invoice)
                 except Exception as e:
-                    _logger.exception("Error importing attachment \"%s\" as invoice with format \"%s\"", file_data['filename'], edi_format.name, str(e))
+                    _logger.exception(
+                        "Error importing attachment \"%s\" as invoice with format \"%s\": %s",
+                        file_data['filename'],
+                        edi_format.name,
+                        str(e))
                 if res:
-                    return res
+                    return res._link_invoice_origin_to_purchase_orders(timeout=4)
         return self.env['account.move']
 
     ####################################################
@@ -502,89 +382,101 @@ class AccountEdiFormat(models.Model):
         element = xml_element.xpath(xpath, namespaces=namespaces)
         return element[0].text if element else None
 
+    @api.model
+    def _retrieve_partner_with_vat(self, vat, extra_domain):
+        if not vat:
+            return None
+
+        # Sometimes, the vat is specified with some whitespaces.
+        normalized_vat = vat.replace(' ', '')
+        country_prefix = re.match('^[a-zA-Z]{2}|^', vat).group()
+
+        partner = self.env['res.partner'].search(extra_domain + [('vat', 'in', (normalized_vat, vat))], limit=1)
+
+        # Try to remove the country code prefix from the vat.
+        if not partner and country_prefix:
+            partner = self.env['res.partner'].search(extra_domain + [
+                ('vat', 'in', (normalized_vat[2:], vat[2:])),
+                ('country_id.code', '=', country_prefix.upper()),
+            ], limit=1)
+
+            # The country could be not specified on the partner.
+            if not partner:
+                partner = self.env['res.partner'].search(extra_domain + [
+                    ('vat', 'in', (normalized_vat[2:], vat[2:])),
+                    ('country_id', '=', False),
+                ], limit=1)
+
+            # The vat could be a string of alphanumeric values without country code but with missing zeros at the
+            # beginning.
+        if not partner:
+            try:
+                vat_only_numeric = str(int(re.sub(r'^\D{2}', '', normalized_vat) or 0))
+            except ValueError:
+                vat_only_numeric = None
+
+            if vat_only_numeric:
+                query = self.env['res.partner']._where_calc(extra_domain + [('active', '=', True)])
+                tables, where_clause, where_params = query.get_sql()
+
+                if country_prefix:
+                    vat_prefix_regex = f'({country_prefix})?'
+                else:
+                    vat_prefix_regex = '([A-z]{2})?'
+
+                self._cr.execute(f'''
+                    SELECT res_partner.id
+                    FROM {tables}
+                    WHERE {where_clause}
+                    AND res_partner.vat ~ %s
+                    LIMIT 1
+                ''', where_params + ['^%s0*%s$' % (vat_prefix_regex, vat_only_numeric)])
+                partner_row = self._cr.fetchone()
+                if partner_row:
+                    partner = self.env['res.partner'].browse(partner_row[0])
+
+        return partner
+
+    @api.model
+    def _retrieve_partner_with_phone_mail(self, phone, mail, extra_domain):
+        domains = []
+        if phone:
+            domains.append([('phone', '=', phone)])
+            domains.append([('mobile', '=', phone)])
+        if mail:
+            domains.append([('email', '=', mail)])
+
+        if not domains:
+            return None
+
+        domain = expression.OR(domains)
+        if extra_domain:
+            domain = expression.AND([domain, extra_domain])
+        return self.env['res.partner'].search(domain, limit=1)
+
+    @api.model
+    def _retrieve_partner_with_name(self, name, extra_domain):
+        if not name:
+            return None
+        return self.env['res.partner'].search([('name', 'ilike', name)] + extra_domain, limit=1)
+
     def _retrieve_partner(self, name=None, phone=None, mail=None, vat=None, domain=None):
         '''Search all partners and find one that matches one of the parameters.
-
         :param name:    The name of the partner.
         :param phone:   The phone or mobile of the partner.
         :param mail:    The mail of the partner.
         :param vat:     The vat number of the partner.
         :returns:       A partner or an empty recordset if not found.
         '''
+
         def search_with_vat(extra_domain):
-            if not vat:
-                return None
-
-            # Sometimes, the vat is specified with some whitespaces.
-            normalized_vat = vat.replace(' ', '')
-            country_prefix = re.match('^[A-Z]{2}|^', vat, re.I).group()
-
-            partner = self.env['res.partner'].search(extra_domain + [('vat', 'in', (normalized_vat, vat))], limit=1)
-
-            # Try to remove the country code prefix from the vat.
-            if not partner and country_prefix:
-                partner = self.env['res.partner'].search(extra_domain + [
-                    ('vat', 'in', (normalized_vat[2:], vat[2:])),
-                    ('country_id.code', '=', country_prefix.upper()),
-                ], limit=1)
-
-                # The country could be not specified on the partner.
-                if not partner:
-                    partner = self.env['res.partner'].search(extra_domain + [
-                        ('vat', 'in', (normalized_vat[2:], vat[2:])),
-                        ('country_id', '=', False),
-                    ], limit=1)
-
-            # The vat could be a string of alphanumeric values without country code but with missing zeros at the
-            # beginning.
-            if not partner:
-                try:
-                    vat_only_numeric = str(int(re.sub(r'^\D{2}', '', normalized_vat) or 0))
-                except ValueError:
-                    vat_only_numeric = None
-
-                if vat_only_numeric:
-                    query = self.env['res.partner']._where_calc(extra_domain + [('active', '=', True)])
-                    tables, where_clause, where_params = query.get_sql()
-
-                    if country_prefix:
-                        vat_prefix_regex = f'({country_prefix})?'
-                    else:
-                        vat_prefix_regex = '([A-Z]{2})?'
-
-                    self._cr.execute(f'''
-                        SELECT res_partner.id
-                        FROM {tables}
-                        WHERE {where_clause}
-                        AND res_partner.vat ~* %s
-                        LIMIT 1
-                    ''', where_params + ['^%s0*%s$' % (vat_prefix_regex, vat_only_numeric)])
-                    partner_row = self._cr.fetchone()
-                    if partner_row:
-                        partner = self.env['res.partner'].browse(partner_row[0])
-
-            return partner
+            return self._retrieve_partner_with_vat(vat, extra_domain)
 
         def search_with_phone_mail(extra_domain):
-            domains = []
-            if phone:
-                domains.append([('phone', '=', phone)])
-                domains.append([('mobile', '=', phone)])
-            if mail:
-                domains.append([('email', '=', mail)])
-
-            if not domains:
-                return None
-
-            domain = expression.OR(domains)
-            if extra_domain:
-                domain = expression.AND([domain, extra_domain])
-            return self.env['res.partner'].search(domain, limit=1)
+            return self._retrieve_partner_with_phone_mail(phone, mail, extra_domain)
 
         def search_with_name(extra_domain):
-            if not name:
-                return None
-            return self.env['res.partner'].search([('name', 'ilike', name)] + extra_domain, limit=1)
+            return self._retrieve_partner_with_name(name, extra_domain)
 
         def search_with_domain(extra_domain):
             if not domain:
@@ -596,7 +488,6 @@ class AccountEdiFormat(models.Model):
                 partner = search_method(extra_domain)
                 if partner:
                     return partner
-
         return self.env['res.partner']
 
     def _retrieve_product(self, name=None, default_code=None, barcode=None):
@@ -607,6 +498,9 @@ class AccountEdiFormat(models.Model):
         :param barcode:         The barcode of the product.
         :returns:               A product or an empty recordset if not found.
         '''
+        if name and '\n' in name:
+            # cut Sales Description from the name
+            name = name.split('\n')[0]
         domains = []
         for value, domain in (
             (name, ('name', 'ilike', name)),
@@ -616,7 +510,10 @@ class AccountEdiFormat(models.Model):
             if value is not None:
                 domains.append([domain])
 
-        domain = expression.OR(domains)
+        domain = expression.AND([
+            expression.OR(domains),
+            [('company_id', 'in', [False, self.env.company.id])],
+        ])
         return self.env['product.product'].search(domain, limit=1)
 
     def _retrieve_tax(self, amount, type_tax_use):
@@ -628,7 +525,8 @@ class AccountEdiFormat(models.Model):
         '''
         domains = [
             [('amount', '=', float(amount))],
-            [('type_tax_use', '=', type_tax_use)]
+            [('type_tax_use', '=', type_tax_use)],
+            [('company_id', '=', self.env.company.id)]
         ]
 
         return self.env['account.tax'].search(expression.AND(domains), order='sequence ASC', limit=1)
@@ -639,7 +537,21 @@ class AccountEdiFormat(models.Model):
         :param code: The code of the currency.
         :returns:    A currency or an empty recordset if not found.
         '''
-        return self.env['res.currency'].with_context(active_test=False).search([('name', '=', code.upper())], limit=1)
+        currency = self.env['res.currency'].with_context(active_test=False).search([('name', '=', code.upper())], limit=1)
+        if currency and not currency.active:
+            error_msg = _('The currency (%s) of the document you are uploading is not active in this database.\n'
+                          'Please activate it and update the currency rate if needed before trying again to import.',
+                          currency.name)
+            error_action = {
+                'view_mode': 'form',
+                'res_model': 'res.currency',
+                'type': 'ir.actions.act_window',
+                'target': 'new',
+                'res_id': currency.id,
+                'views': [[False, 'form']]
+            }
+            raise RedirectWarning(error_msg, error_action, _('Display the currency'))
+        return currency
 
     ####################################################
     # Other helpers
