@@ -9,7 +9,6 @@ import time
 _logger = logging.getLogger(__name__)
 
 
-@tagged('external_l10n', '-at_install', 'post_install', '-standard', 'external')
 class TestAr(AccountTestInvoicingCommon):
 
     @classmethod
@@ -322,6 +321,9 @@ class TestAr(AccountTestInvoicingCommon):
         invoice_user_id = self.env.user
         incoterm = self.env.ref("account.incoterm_EXW")
 
+        decimal_price = self.env.ref('product.decimal_price')
+        decimal_price.digits = 4
+
         invoices_to_create = {
             'test_invoice_1': {
                 "ref": "test_invoice_1: Invoice to gritti support service, vat 21",
@@ -636,7 +638,7 @@ class TestAr(AccountTestInvoicingCommon):
         values.update(data)
 
         journal = self.env['account.journal'].create(values)
-        _logger.info('Created journal %s for company %s' % (journal.name, self.env.company.name))
+        _logger.info('Created journal %s for company %s', journal.name, self.env.company.name)
         return journal
 
     def _create_invoice(self, data=None, invoice_type='out_invoice'):
@@ -656,8 +658,8 @@ class TestAr(AccountTestInvoicingCommon):
                 invoice_form.currency_id = data.get('currency')
             for line in data.get('lines', [{}]):
                 with invoice_form.invoice_line_ids.new() as invoice_line_form:
-                    if line.get('display_type'):
-                        invoice_line_form.display_type = line.get('display_type')
+                    invoice_line_form.display_type = line.get('display_type', 'product')
+                    if line.get('display_type') in ('line_note', 'line_section'):
                         invoice_line_form.name = line.get('name', 'not invoice line')
                     else:
                         invoice_line_form.product_id = line.get('product', self.product_iva_21)
@@ -744,4 +746,3 @@ class TestAr(AccountTestInvoicingCommon):
             rate.rate = value
         else:
             rate_obj.create({'company_id': self.env.company.id, 'currency_id': currency.id, 'rate': value})
-
