@@ -5,6 +5,7 @@ import base64
 import re
 from odoo import api, fields, models, _
 from odoo.exceptions import UserError, ValidationError
+from odoo.tools import float_round, float_repr
 
 FK_HEAD_LIST = ['FK', 'KD_JENIS_TRANSAKSI', 'FG_PENGGANTI', 'NOMOR_FAKTUR', 'MASA_PAJAK', 'TAHUN_PAJAK', 'TANGGAL_FAKTUR', 'NPWP', 'NAMA', 'ALAMAT_LENGKAP', 'JUMLAH_DPP', 'JUMLAH_PPN', 'JUMLAH_PPNBM', 'ID_KETERANGAN_TAMBAHAN', 'FG_UANG_MUKA', 'UANG_MUKA_DPP', 'UANG_MUKA_PPN', 'UANG_MUKA_PPNBM', 'REFERENSI', 'KODE_DOKUMEN_PENDUKUNG']
 
@@ -29,19 +30,16 @@ class AccountMove(models.Model):
             ('02', '02 Kepada Pemungut Bendaharawan (Dinas Kepemerintahan)'),
             ('03', '03 Kepada Pemungut Selain Bendaharawan (BUMN)'),
             ('04', '04 DPP Nilai Lain (PPN 1%)'),
+            ('05', '05 Besaran Tertentu'),
             ('06', '06 Penyerahan Lainnya (Turis Asing)'),
             ('07', '07 Penyerahan yang PPN-nya Tidak Dipungut (Kawasan Ekonomi Khusus/ Batam)'),
             ('08', '08 Penyerahan yang PPN-nya Dibebaskan (Impor Barang Tertentu)'),
             ('09', '09 Penyerahan Aktiva ( Pasal 16D UU PPN )'),
         ], string='Kode Transaksi', help='Dua digit pertama nomor pajak',
-        readonly=True, states={'draft': [('readonly', False)]}, copy=False)
+        readonly=False, states={'posted': [('readonly', True)], 'cancel': [('readonly', True)]}, copy=False,
+        compute="_compute_kode_transaksi", store=True)
     l10n_id_need_kode_transaksi = fields.Boolean(compute='_compute_need_kode_transaksi')
 
-    @api.onchange('partner_id')
-    def _onchange_partner_id(self):
-        self.l10n_id_kode_transaksi = self.partner_id.l10n_id_kode_transaksi
-        return super(AccountMove, self)._onchange_partner_id()
-
     @api.onchange('l10n_id_tax_number')
     def _onchange_l10n_id_tax_number(self):
         for record in self:
@@ -53,20 +51,25 @@ class AccountMove(models.Model):
         for record in self:
             record.l10n_id_csv_created = bool(record.l10n_id_attachment_id)
 
+    @api.depends('partner_id')
+    def _compute_kode_transaksi(self):
+        for move in self:
+            move.l10n_id_kode_transaksi = move.partner_id.l10n_id_kode_transaksi
+
     @api.depends('partner_id')
     def _compute_need_kode_transaksi(self):
         for move in self:
             move.l10n_id_need_kode_transaksi = move.partner_id.l10n_id_pkp and not move.l10n_id_tax_number and move.move_type == 'out_invoice' and move.country_code == 'ID'
 
-    @api.constrains('l10n_id_kode_transaksi', 'line_ids')
+    @api.constrains('l10n_id_kode_transaksi', 'line_ids', 'partner_id')
     def _constraint_kode_ppn(self):
         ppn_tag = self.env.ref('l10n_id.ppn_tag')
-        for move in self.filtered(lambda m: m.l10n_id_kode_transaksi != '08'):
-            if any(ppn_tag.id in line.tax_tag_ids.ids for line in move.line_ids if line.exclude_from_invoice_tab is False and not line.display_type) \
-                    and any(ppn_tag.id not in line.tax_tag_ids.ids for line in move.line_ids if line.exclude_from_invoice_tab is False and not line.display_type):
+        for move in self.filtered(lambda m: m.l10n_id_need_kode_transaksi and m.l10n_id_kode_transaksi != '08'):
+            if any(ppn_tag.id in line.tax_tag_ids.ids for line in move.line_ids if line.display_type == 'product') \
+                    and any(ppn_tag.id not in line.tax_tag_ids.ids for line in move.line_ids if line.display_type == 'product'):
                 raise UserError(_('Cannot mix VAT subject and Non-VAT subject items in the same invoice with this kode transaksi.'))
-        for move in self.filtered(lambda m: m.l10n_id_kode_transaksi == '08'):
-            if any(ppn_tag.id in line.tax_tag_ids.ids for line in move.line_ids if line.exclude_from_invoice_tab is False and not line.display_type):
+        for move in self.filtered(lambda m: m.l10n_id_need_kode_transaksi and m.l10n_id_kode_transaksi == '08'):
+            if any(ppn_tag.id in line.tax_tag_ids.ids for line in move.line_ids if line.display_type == 'product'):
                 raise UserError('Kode transaksi 08 is only for non VAT subject items.')
 
     @api.constrains('l10n_id_tax_number')
@@ -134,8 +137,6 @@ class AccountMove(models.Model):
     def _generate_efaktur_invoice(self, delimiter):
         """Generate E-Faktur for customer invoice."""
         # Invoice of Customer
-        company_id = self.company_id
-        dp_product_id = self.env['ir.config_parameter'].sudo().get_param('sale.default_deposit_product_id')
 
         output_head = '%s%s%s' % (
             _csv_row(FK_HEAD_LIST, delimiter),
@@ -143,6 +144,8 @@ class AccountMove(models.Model):
             _csv_row(OF_HEAD_LIST, delimiter),
         )
 
+        idr = self.env.ref('base.IDR')
+
         for move in self.filtered(lambda m: m.state == 'posted'):
             eTax = move._prepare_etax()
 
@@ -172,52 +175,47 @@ class AccountMove(models.Model):
             eTax['NPWP'] = invoice_npwp
             eTax['NAMA'] = move.partner_id.name if eTax['NPWP'] == '000000000000000' else move.partner_id.l10n_id_tax_name or move.partner_id.name
             eTax['ALAMAT_LENGKAP'] = move.partner_id.contact_address.replace('\n', '') if eTax['NPWP'] == '000000000000000' else move.partner_id.l10n_id_tax_address or street
-            eTax['JUMLAH_DPP'] = int(round(move.amount_untaxed, 0)) # currency rounded to the unit
-            eTax['JUMLAH_PPN'] = int(round(move.amount_tax, 0))
+            eTax['JUMLAH_DPP'] = int(float_round(move.amount_untaxed, 0, rounding_method="DOWN"))  # currency rounded to the unit
+            eTax['JUMLAH_PPN'] = int(float_round(move.amount_tax, 0, rounding_method="DOWN"))  # tax amount ALWAYS rounded down
             eTax['ID_KETERANGAN_TAMBAHAN'] = '1' if move.l10n_id_kode_transaksi == '07' else ''
             eTax['REFERENSI'] = number_ref
             eTax['KODE_DOKUMEN_PENDUKUNG'] = '0'
 
-            lines = move.line_ids.filtered(lambda x: x.product_id.id == int(dp_product_id) and x.price_unit < 0 and not x.display_type)
+            lines = move.line_ids.filtered(lambda x: x.move_id._is_downpayment() and x.price_unit < 0 and x.display_type == 'product')
             eTax['FG_UANG_MUKA'] = 0
-            eTax['UANG_MUKA_DPP'] = int(abs(sum(lines.mapped(lambda l: round(l.price_subtotal, 0)))))
-            eTax['UANG_MUKA_PPN'] = int(abs(sum(lines.mapped(lambda l: round(l.price_total - l.price_subtotal, 0)))))
-
-            company_npwp = company_id.partner_id.vat or '000000000000000'
+            eTax['UANG_MUKA_DPP'] = float_repr(abs(sum(lines.mapped(lambda l: float_round(l.price_subtotal, 0)))), 0)
+            eTax['UANG_MUKA_PPN'] = float_repr(abs(sum(lines.mapped(lambda l: float_round(l.price_total - l.price_subtotal, 0)))), 0)
 
             fk_values_list = ['FK'] + [eTax[f] for f in FK_HEAD_LIST[1:]]
-            eTax['JALAN'] = company_id.partner_id.l10n_id_tax_address or company_id.partner_id.street
-            eTax['NOMOR_TELEPON'] = company_id.phone or ''
-
-            lt_values_list = ['FAPR', company_npwp, company_id.name] + [eTax[f] for f in LT_HEAD_LIST[3:]]
 
             # HOW TO ADD 2 line to 1 line for free product
             free, sales = [], []
 
-            for line in move.line_ids.filtered(lambda l: not l.exclude_from_invoice_tab and not l.display_type):
+            for line in move.line_ids.filtered(lambda l: l.display_type == 'product'):
                 # *invoice_line_unit_price is price unit use for harga_satuan's column
                 # *invoice_line_quantity is quantity use for jumlah_barang's column
                 # *invoice_line_total_price is bruto price use for harga_total's column
                 # *invoice_line_discount_m2m is discount price use for diskon's column
                 # *line.price_subtotal is subtotal price use for dpp's column
                 # *tax_line or free_tax_line is tax price use for ppn's column
-                free_tax_line = tax_line = bruto_total = total_discount = 0.0
+                free_tax_line = tax_line = 0.0
 
                 for tax in line.tax_ids:
                     if tax.amount > 0:
                         tax_line += line.price_subtotal * (tax.amount / 100.0)
 
-                invoice_line_unit_price = line.price_unit
-
-                invoice_line_total_price = invoice_line_unit_price * line.quantity
+                discount = 1 - (line.discount / 100)
+                # guarantees price to be tax-excluded
+                invoice_line_total_price = line.price_subtotal / discount if discount else 0
+                invoice_line_unit_price = invoice_line_total_price / line.quantity if line.quantity else 0
 
                 line_dict = {
                     'KODE_OBJEK': line.product_id.default_code or '',
                     'NAMA': line.product_id.name or '',
-                    'HARGA_SATUAN': int(round(invoice_line_unit_price, 0)),
+                    'HARGA_SATUAN': float_repr(idr.round(invoice_line_unit_price), idr.decimal_places),
                     'JUMLAH_BARANG': line.quantity,
-                    'HARGA_TOTAL': int(round(invoice_line_total_price, 0)),
-                    'DPP': int(round(line.price_subtotal, 0)),
+                    'HARGA_TOTAL': idr.round(invoice_line_total_price),
+                    'DPP': line.price_subtotal,
                     'product_id': line.product_id.id,
                 }
 
@@ -226,16 +224,16 @@ class AccountMove(models.Model):
                         free_tax_line += (line.price_subtotal * (tax.amount / 100.0)) * -1.0
 
                     line_dict.update({
-                        'DISKON': int(round(invoice_line_total_price - line.price_subtotal, 0)),
-                        'PPN': int(round(free_tax_line, 0)),
+                        'DISKON': float_round(invoice_line_total_price - line.price_subtotal, 0),
+                        'PPN': free_tax_line,
                     })
                     free.append(line_dict)
                 elif line.price_subtotal != 0.0:
                     invoice_line_discount_m2m = invoice_line_total_price - line.price_subtotal
 
                     line_dict.update({
-                        'DISKON': int(round(invoice_line_discount_m2m, 0)),
-                        'PPN': int(round(tax_line, 0)),
+                        'DISKON': float_round(invoice_line_discount_m2m, 0),
+                        'PPN': tax_line,
                     })
                     sales.append(line_dict)
 
@@ -258,17 +256,60 @@ class AccountMove(models.Model):
                             if tax.amount > 0:
                                 tax_line += sale['DPP'] * (tax.amount / 100.0)
 
-                        sale['PPN'] = int(round(tax_line, 0))
+                        sale['PPN'] = tax_line
 
                         free.remove(f)
 
                 sub_total_before_adjustment += sale['DPP']
                 sub_total_ppn_before_adjustment += sale['PPN']
-                bruto_total += sale['DISKON']
-                total_discount += round(sale['DISKON'], 2)
+
+                sale.update({
+                    # Use the db currency rounding to float_round the DPP/PPN.
+                    # As we will correct them we need them to be close to the final result.
+                    'DPP': idr.round(sale['DPP']),
+                    'PPN': idr.round(sale['PPN']),
+                    'DISKON': float_repr(sale['DISKON'], 0),
+                })
+
+
+            # The total of the base (DPP) and taxes (PPN) must be a integer, equal to the JUMLAH_DPP and JUMLAH_PPN
+            # To do so, we adjust the first line in order to achieve the correct total
+            if sales:
+                diff_dpp = idr.round(eTax['JUMLAH_DPP'] - sum([sale['DPP'] for sale in sales]))
+                total_sales_ppn = idr.round(eTax['JUMLAH_PPN'] - sum([sale['PPN'] for sale in sales]))
+                # We will add the differences to the first line for which adding the difference will not result in a negative value.
+                for sale in sales:
+                    if sale['DPP'] + diff_dpp >= 0 and sale['PPN'] + total_sales_ppn >= 0:
+                        sale['HARGA_TOTAL'] += diff_dpp
+                        sale['DPP'] += diff_dpp
+                        diff_dpp = 0
+                        sale['PPN'] += total_sales_ppn
+                        total_sales_ppn = 0
+                        break
+
+                # We couldn't adjust everything in a single line as their values is too low.
+                # So we will instead slit the adjustment in multiple lines.
+                if diff_dpp or total_sales_ppn:
+                    for sale in sales:
+                        # DPP
+                        sale_dpp = sale['DPP']
+                        sale["DPP"] = max(0, sale["DPP"] + diff_dpp)
+                        diff_dpp -= (sale["DPP"] - sale_dpp)
+                        sale['HARGA_TOTAL'] = sale["DPP"]
+                        # PPN
+                        sale_ppn = sale['PPN']
+                        sale["PPN"] = max(0, sale["PPN"] + total_sales_ppn)
+                        total_sales_ppn -= (sale["PPN"] - sale_ppn)
+
+            # Values now being corrected, we can format them for the CSV
+            for sale in sales:
+                sale.update({
+                    'HARGA_TOTAL': float_repr(sale['HARGA_TOTAL'], idr.decimal_places),
+                    'DPP': float_repr(sale['DPP'], idr.decimal_places),
+                    'PPN': float_repr(sale['PPN'], idr.decimal_places),
+                })
 
             output_head += _csv_row(fk_values_list, delimiter)
-            output_head += _csv_row(lt_values_list, delimiter)
             for sale in sales:
                 of_values_list = ['OF'] + [str(sale[f]) for f in OF_HEAD_LIST[1:-2]] + ['0', '0']
                 output_head += _csv_row(of_values_list, delimiter)
@@ -277,7 +318,7 @@ class AccountMove(models.Model):
 
     def _prepare_etax(self):
         # These values are never set
-        return {'JUMLAH_PPNBM': 0, 'UANG_MUKA_PPNBM': 0, 'BLOK': '', 'NOMOR': '', 'RT': '', 'RW': '', 'KECAMATAN': '', 'KELURAHAN': '', 'KABUPATEN': '', 'PROPINSI': '', 'KODE_POS': '', 'JUMLAH_BARANG': 0, 'TARIF_PPNBM': 0, 'PPNBM': 0}
+        return {'JUMLAH_PPNBM': 0, 'UANG_MUKA_PPNBM': 0, 'JUMLAH_BARANG': 0, 'TARIF_PPNBM': 0, 'PPNBM': 0}
 
     def _generate_efaktur(self, delimiter):
         if self.filtered(lambda x: not x.l10n_id_kode_transaksi):
