@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import test_login
+from . import test_message_format_portal
 from . import test_tours
 from . import test_portal_wizard
