@@ -16,14 +16,13 @@ discount to a customer.
 """,
     'depends': ['point_of_sale'],
     'data': [
-        'views/pos_discount_views.xml',
+        'views/res_config_settings_views.xml',
+        'views/pos_config_views.xml',
         ],
     'installable': True,
     'assets': {
         'point_of_sale.assets': [
             'pos_discount/static/src/js/**/*',
-        ],
-        'web.assets_qweb': [
             'pos_discount/static/src/xml/**/*',
         ],
     },
