@@ -32,35 +32,13 @@ class L10nLatamDocumentType(models.Model):
             ('I', 'I'),  # used for mapping of imports
         ]
 
-    def _get_document_sequence_vals(self, journal):
-        """ Values to create the sequences """
-        values = super()._get_document_sequence_vals(journal)
-        if self.country_id != self.env.ref('base.ar'):
-            return values
-
-        values.update({'padding': 8, 'implementation': 'no_gap', 'prefix': "%05i-" % (journal.l10n_ar_afip_pos_number),
-                       'l10n_latam_journal_id': journal.id})
-        if journal.l10n_ar_share_sequences:
-            values.update({'name': _('%s - Letter %s Documents') % (journal.name, self.l10n_ar_letter),
-                           'l10n_ar_letter': self.l10n_ar_letter})
-        else:
-            values.update({'name': '%s - %s' % (journal.name, self.name), 'l10n_latam_document_type_id': self.id})
-        return values
-
-    def _filter_taxes_included(self, taxes):
-        """ In argentina we include taxes depending on document letter """
-        self.ensure_one()
-        if self.country_id == self.env.ref('base.ar') and self.l10n_ar_letter in ['B', 'C', 'X', 'R']:
-            return taxes.filtered(lambda x: x.tax_group_id.l10n_ar_vat_afip_code)
-        return super()._filter_taxes_included(taxes)
-
     def _format_document_number(self, document_number):
         """ Make validation of Import Dispatch Number
           * making validations on the document_number. If it is wrong it should raise an exception
           * format the document_number against a pattern and return it
         """
         self.ensure_one()
-        if self.country_id != self.env.ref('base.ar'):
+        if self.country_id.code != "AR":
             return super()._format_document_number(document_number)
 
         if not document_number:
