@@ -1,18 +1,24 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 import re
 
 from odoo import models, fields, api, _
-from odoo.exceptions import ValidationError, UserError
+from odoo.exceptions import UserError
 from odoo.tools.float_utils import float_split_str
 from odoo.tools.misc import mod10r
 
-
 l10n_ch_ISR_NUMBER_LENGTH = 27
 l10n_ch_ISR_ID_NUM_LENGTH = 6
 
 class AccountMove(models.Model):
+    # NOTE
+    # The ISR system is kept and taken into account up to September 2022.
+    # After that, the transition to the QR system will be completed and the ISR system won't exist anymore.
+    # This means that Odoo v16 shouldn't support the ISR system and all the references to it should be cleaned up by then.
+    # In the versions leading to that change,
+    # although the functions related to the ISR are still taken into account and still exist,
+    # the QR billing is always preferred.
+
     _inherit = 'account.move'
 
     l10n_ch_isr_subscription = fields.Char(compute='_compute_l10n_ch_isr_subscription', help='ISR subscription number identifying your company or your bank to generate ISR.')
@@ -29,6 +35,14 @@ class AccountMove(models.Model):
     l10n_ch_currency_name = fields.Char(related='currency_id.name', readonly=True, string="Currency Name", help="The name of this invoice's currency") #This field is used in the "invisible" condition field of the 'Print ISR' button.
     l10n_ch_isr_needs_fixing = fields.Boolean(compute="_compute_l10n_ch_isr_needs_fixing", help="Used to show a warning banner when the vendor bill needs a correct ISR payment reference. ")
 
+    l10n_ch_is_qr_valid = fields.Boolean(compute='_compute_l10n_ch_qr_is_valid', help="Determines whether an invoice can be printed as a QR or not")
+
+    @api.depends('partner_id', 'currency_id')
+    def _compute_l10n_ch_qr_is_valid(self):
+        for move in self:
+            move.l10n_ch_is_qr_valid = move.move_type == 'out_invoice' \
+                                       and move.partner_bank_id._eligible_for_qr_code('ch_qr', move.partner_id, move.currency_id, raises_error=False)
+
     @api.depends('partner_bank_id.l10n_ch_isr_subscription_eur', 'partner_bank_id.l10n_ch_isr_subscription_chf')
     def _compute_l10n_ch_isr_subscription(self):
         """ Computes the ISR subscription identifying your company or the bank that allows to generate ISR. And formats it accordingly"""
@@ -69,11 +83,19 @@ class AccountMove(models.Model):
         # number for Vendor bank accounts:
         # - validation of format xx-yyyyy-c
         # - validation of checksum
-        self.ensure_one()
         return self.partner_bank_id.l10n_ch_postal or ''
 
     @api.depends('name', 'partner_bank_id.l10n_ch_postal')
     def _compute_l10n_ch_isr_number(self):
+        for record in self:
+            if (record.partner_bank_id.l10n_ch_qr_iban or record.l10n_ch_isr_subscription) and record.name:
+                invoice_ref = re.sub(r'\D', '', record.name)
+                record.l10n_ch_isr_number = record._compute_isr_number(invoice_ref)
+            else:
+                record.l10n_ch_isr_number = False
+
+    @api.model
+    def _compute_isr_number(self, invoice_ref):
         """Generates the ISR or QRR reference
 
         An ISR references are 27 characters long.
@@ -118,22 +140,18 @@ class AccountMove(models.Model):
                 (2) 12345678901234567890 | reference
                 (3) 1: control digit for identification number and reference
         """
-        for record in self:
-            if (record.partner_bank_id.l10n_ch_qr_iban or record.l10n_ch_isr_subscription) and record.name:
-                id_number = record._get_isrb_id_number()
-                if id_number:
-                    id_number = id_number.zfill(l10n_ch_ISR_ID_NUM_LENGTH)
-                invoice_ref = re.sub('[^\d]', '', record.name)
-                # keep only the last digits if it exceed boundaries
-                full_len = len(id_number) + len(invoice_ref)
-                ref_payload_len = l10n_ch_ISR_NUMBER_LENGTH - 1
-                extra = full_len - ref_payload_len
-                if extra > 0:
-                    invoice_ref = invoice_ref[extra:]
-                internal_ref = invoice_ref.zfill(ref_payload_len - len(id_number))
-                record.l10n_ch_isr_number = mod10r(id_number + internal_ref)
-            else:
-                record.l10n_ch_isr_number = False
+        id_number = self._get_isrb_id_number()
+        if id_number:
+            id_number = id_number.zfill(l10n_ch_ISR_ID_NUM_LENGTH)
+        # keep only the last digits if it exceed boundaries
+        full_len = len(id_number) + len(invoice_ref)
+        ref_payload_len = l10n_ch_ISR_NUMBER_LENGTH - 1
+        extra = full_len - ref_payload_len
+        if extra > 0:
+            invoice_ref = invoice_ref[extra:]
+        internal_ref = invoice_ref.zfill(ref_payload_len - len(id_number))
+
+        return mod10r(id_number + internal_ref)
 
     @api.depends('l10n_ch_isr_number')
     def _compute_l10n_ch_isr_number_spaced(self):
@@ -242,7 +260,7 @@ class AccountMove(models.Model):
     @api.depends('move_type', 'partner_bank_id', 'payment_reference')
     def _compute_l10n_ch_isr_needs_fixing(self):
         for inv in self:
-            if inv.move_type == 'in_invoice' and inv.company_id.account_fiscal_country_id.code == "CH":
+            if inv.move_type == 'in_invoice' and inv.company_id.account_fiscal_country_id.code in ('CH', 'LI'):
                 partner_bank = inv.partner_bank_id
                 needs_isr_ref = partner_bank.l10n_ch_qr_iban or partner_bank._is_isr_issuer()
                 if needs_isr_ref and not inv._has_isr_ref():
@@ -278,42 +296,11 @@ class AccountMove(models.Model):
         """
         return float_split_str(self.amount_residual, 2)
 
-    def isr_print(self):
-        """ Triggered by the 'Print ISR' button.
-        """
-        self.ensure_one()
-        if self.l10n_ch_isr_valid:
-            self.l10n_ch_isr_sent = True
-            return self.env.ref('l10n_ch.l10n_ch_isr_report').report_action(self)
-        else:
-            raise ValidationError(_("""You cannot generate an ISR yet.\n
-                                   For this, you need to :\n
-                                   - set a valid postal account number (or an IBAN referencing one) for your company\n
-                                   - define its bank\n
-                                   - associate this bank with a postal reference for the currency used in this invoice\n
-                                   - fill the 'bank account' field of the invoice with the postal to be used to receive the related payment. A default account will be automatically set for all invoices created after you defined a postal account for your company."""))
-
-    def print_ch_qr_bill(self):
-        """ Triggered by the 'Print QR-bill' button.
-        """
-        self.ensure_one()
-
-        if not self.partner_bank_id:
-            raise UserError(_("QR-Bill can not be generated on paid invoices. If the invoice is not fully paid, please make sure Recipient Bank field is not empty and try again."))
-
-        if not self.partner_bank_id._eligible_for_qr_code('ch_qr', self.partner_id, self.currency_id):
-            raise UserError(_("Cannot generate the QR-bill. Please check you have configured the address of your company and debtor. If you are using a QR-IBAN, also check the invoice's payment reference is a QR reference."))
-
-        self.l10n_ch_isr_sent = True
-        return self.env.ref('l10n_ch.l10n_ch_qr_report').report_action(self)
-
     def action_invoice_sent(self):
         # OVERRIDE
         rslt = super(AccountMove, self).action_invoice_sent()
-
-        if self.l10n_ch_isr_valid:
+        if self.l10n_ch_isr_valid or self.l10n_ch_is_qr_valid:
             rslt['context']['l10n_ch_mark_isr_as_sent'] = True
-
         return rslt
 
     @api.returns('mail.message', lambda value: value.id)
@@ -327,6 +314,9 @@ class AccountMove(models.Model):
         `Payment Reference` of the invoice when invoice's journal is using Switzerland's communication standard
         """
         self.ensure_one()
+        # l10n_ch_isr_number is not always computed at this stage, and could change value when the invoice is posted.
+        # We manually compute here it to avoid this conflict.
+        self._compute_l10n_ch_isr_number()
         return self.l10n_ch_isr_number
 
     def _get_invoice_reference_ch_partner(self):
@@ -346,5 +336,41 @@ class AccountMove(models.Model):
         while i > 0:
             spaced_qrr_ref = qrr_ref[max(i-5, 0) : i] + ' ' + spaced_qrr_ref
             i -= 5
-
         return spaced_qrr_ref
+
+    @api.model
+    def space_scor_reference(self, iso11649_ref):
+        """ Makes the provided SCOR reference human-friendly, spacing its elements
+        by blocks of 5 from right to left.
+        """
+
+        return ' '.join(iso11649_ref[i:i + 4] for i in range(0, len(iso11649_ref), 4))
+
+    def l10n_ch_action_print_qr(self):
+        '''
+        Checks that all invoices can be printed in the QR format.
+        If so, launches the printing action.
+        Else, triggers the l10n_ch wizard that will display the informations.
+        '''
+        if any(x.move_type != 'out_invoice' for x in self):
+            raise UserError(_("Only customers invoices can be QR-printed."))
+        if False in self.mapped('l10n_ch_is_qr_valid'):
+            return {
+                'name': (_("Some invoices could not be printed in the QR format")),
+                'type': 'ir.actions.act_window',
+                'res_model': 'l10n_ch.qr_invoice.wizard',
+                'view_type': 'form',
+                'view_mode': 'form',
+                'target': 'new',
+                'context': {'active_ids': self.ids},
+            }
+        return self.env.ref('account.account_invoices').report_action(self)
+
+    def _l10n_ch_dispatch_invoices_to_print(self):
+        qr_invs = self.filtered('l10n_ch_is_qr_valid')
+        isr_invs = self.filtered('l10n_ch_isr_valid')
+        return {
+            'qr': qr_invs,
+            'isr': isr_invs,
+            'classic': self - qr_invs - isr_invs,
+        }
