@@ -2,7 +2,7 @@
 {
     'name': 'LATAM Localization Base',
     'version': '1.0',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations',
     'sequence': 14,
     'author': 'Odoo, ADHOC SA',
     'summary': 'LATAM Identification Types',
@@ -56,8 +56,6 @@ This module is compatible with base_vat module in order to be able to validate V
         'security/ir.model.access.csv',
     ],
     'installable': True,
-    'auto_install': False,
-    'application': False,
     'post_init_hook': '_set_default_identification_type',
     'license': 'LGPL-3',
 }
