@@ -8,31 +8,31 @@ from odoo.addons.stock.tests import common2
 class TestMrpCommon(common2.TestStockCommon):
 
     @classmethod
-    def generate_mo(self, tracking_final='none', tracking_base_1='none', tracking_base_2='none', qty_final=5, qty_base_1=4, qty_base_2=1, picking_type_id=False, consumption=False):
+    def generate_mo(cls, tracking_final='none', tracking_base_1='none', tracking_base_2='none', qty_final=5, qty_base_1=4, qty_base_2=1, picking_type_id=False, consumption=False):
         """ This function generate a manufacturing order with one final
         product and two consumed product. Arguments allows to choose
         the tracking/qty for each different products. It returns the
         MO, used bom and the tree products.
         """
-        product_to_build = self.env['product.product'].create({
+        product_to_build = cls.env['product.product'].create({
             'name': 'Young Tom',
             'type': 'product',
             'tracking': tracking_final,
         })
-        product_to_use_1 = self.env['product.product'].create({
+        product_to_use_1 = cls.env['product.product'].create({
             'name': 'Botox',
             'type': 'product',
             'tracking': tracking_base_1,
         })
-        product_to_use_2 = self.env['product.product'].create({
+        product_to_use_2 = cls.env['product.product'].create({
             'name': 'Old Tom',
             'type': 'product',
             'tracking': tracking_base_2,
         })
-        bom_1 = self.env['mrp.bom'].create({
+        bom_1 = cls.env['mrp.bom'].create({
             'product_id': product_to_build.id,
             'product_tmpl_id': product_to_build.product_tmpl_id.id,
-            'product_uom_id': self.uom_unit.id,
+            'product_uom_id': cls.uom_unit.id,
             'product_qty': 1.0,
             'type': 'normal',
             'consumption': consumption if consumption else 'flexible',
@@ -40,7 +40,7 @@ class TestMrpCommon(common2.TestStockCommon):
                 (0, 0, {'product_id': product_to_use_2.id, 'product_qty': qty_base_2}),
                 (0, 0, {'product_id': product_to_use_1.id, 'product_qty': qty_base_1})
             ]})
-        mo_form = Form(self.env['mrp.production'])
+        mo_form = Form(cls.env['mrp.production'])
         mo_form.product_id = product_to_build
         if picking_type_id:
             mo_form.picking_type_id = picking_type_id
@@ -54,6 +54,23 @@ class TestMrpCommon(common2.TestStockCommon):
     def setUpClass(cls):
         super(TestMrpCommon, cls).setUpClass()
 
+        (
+            cls.product_4,
+            cls.product_5,
+            cls.product_6,
+            cls.product_8,
+        ) = cls.env['product.product'].create([{
+            'name': 'Stick',  # product_4
+            'uom_id': cls.uom_dozen.id,
+            'uom_po_id': cls.uom_dozen.id,
+        }, {
+            'name': 'Stone Tools',  # product_5
+        }, {
+            'name': 'Door',  # product_6
+        }, {
+            'name': 'House',  # product_8
+        }])
+
         # Update demo products
         (cls.product_2 | cls.product_3 | cls.product_4 | cls.product_5 | cls.product_6 | cls.product_7_3 | cls.product_8).write({
             'type': 'product',
@@ -66,7 +83,7 @@ class TestMrpCommon(common2.TestStockCommon):
             login='hilda',
             email='h.h@example.com',
             notification_type='inbox',
-            groups='mrp.group_mrp_user, stock.group_stock_user, mrp.group_mrp_byproducts',
+            groups='mrp.group_mrp_user, stock.group_stock_user, mrp.group_mrp_byproducts, uom.group_uom',
         )
         cls.user_mrp_manager = mail_new_test_user(
             cls.env,
@@ -74,26 +91,29 @@ class TestMrpCommon(common2.TestStockCommon):
             login='gary',
             email='g.g@example.com',
             notification_type='inbox',
-            groups='mrp.group_mrp_manager, stock.group_stock_user, mrp.group_mrp_byproducts',
+            groups='mrp.group_mrp_manager, stock.group_stock_user, mrp.group_mrp_byproducts, uom.group_uom',
         )
+        # Required for `product_uom_id` to be visible in the view
+        # This class is used by a lot of tests which sets `product_uom_id` on `mrp.production`
+        cls.env.user.groups_id += cls.env.ref('uom.group_uom')
 
         cls.workcenter_1 = cls.env['mrp.workcenter'].create({
             'name': 'Nuclear Workcenter',
-            'capacity': 2,
+            'default_capacity': 2,
             'time_start': 10,
             'time_stop': 5,
             'time_efficiency': 80,
         })
         cls.workcenter_2 = cls.env['mrp.workcenter'].create({
             'name': 'Simple Workcenter',
-            'capacity': 1,
+            'default_capacity': 1,
             'time_start': 0,
             'time_stop': 0,
             'time_efficiency': 100,
         })
         cls.workcenter_3 = cls.env['mrp.workcenter'].create({
             'name': 'Double Workcenter',
-            'capacity': 2,
+            'default_capacity': 2,
             'time_start': 0,
             'time_stop': 0,
             'time_efficiency': 100,
@@ -208,3 +228,32 @@ class TestMrpCommon(common2.TestStockCommon):
             'tracking': 'none',
             'categ_id': cls.env.ref('product.product_category_all').id,
         })
+
+    @classmethod
+    def make_prods(cls, n):
+        return [
+            cls.env["product.product"].create(
+                {"name": f"p{k + 1}", "type": "product"}
+            )
+            for k in range(n)
+        ]
+
+    @classmethod
+    def make_bom(cls, p, *cs):
+        return cls.env["mrp.bom"].create(
+            {
+                "product_tmpl_id": p.product_tmpl_id.id,
+                "product_id": p.id,
+                "product_qty": 1,
+                "type": "phantom",
+                "product_uom_id": cls.uom_unit.id,
+                "bom_line_ids": [
+                    (0, 0, {
+                        "product_id": c.id,
+                        "product_qty": 1,
+                        "product_uom_id": cls.uom_unit.id
+                    })
+                    for c in cs
+                ],
+            }
+        )
