@@ -17,6 +17,7 @@ The actual till still requires one user but an unlimited number of employees can
         'views/pos_config.xml',
         'views/pos_order_view.xml',
         'views/pos_order_report_view.xml',
+        'views/res_config_settings_views.xml',
     ],
     'installable': True,
     'auto_install': True,
@@ -24,18 +25,17 @@ The actual till still requires one user but an unlimited number of employees can
         'point_of_sale.assets': [
             'pos_hr/static/src/css/pos.css',
             'pos_hr/static/src/js/models.js',
-            'pos_hr/static/src/js/useSelectEmployee.js',
+            'pos_hr/static/src/js/SelectCashierMixin.js',
             'pos_hr/static/src/js/Chrome.js',
             'pos_hr/static/src/js/HeaderLockButton.js',
             'pos_hr/static/src/js/CashierName.js',
             'pos_hr/static/src/js/LoginScreen.js',
+            'pos_hr/static/src/js/PaymentScreen.js',
+            'pos_hr/static/src/xml/**/*',
         ],
         'web.assets_tests': [
             'pos_hr/static/tests/**/*',
         ],
-        'web.assets_qweb': [
-            'pos_hr/static/src/xml/**/*',
-        ],
     },
     'license': 'LGPL-3',
 }
