@@ -71,7 +71,10 @@ class SmsSms(models.Model):
                 notifications.mail_message_id._notify_message_notification_update()
 
     def action_set_outgoing(self):
-        self.state = 'outgoing'
+        self.write({
+            'state': 'outgoing',
+            'failure_type': False
+        })
         notifications = self.env['mail.notification'].sudo().search([
             ('sms_id', 'in', self.ids),
             # sent is sent -> cannot reset
@@ -93,7 +96,7 @@ class SmsSms(models.Model):
         for batch_ids in self._split_batch():
             self.browse(batch_ids)._send(unlink_failed=unlink_failed, unlink_sent=unlink_sent, raise_exception=raise_exception)
             # auto-commit if asked except in testing mode
-            if auto_commit is True and not getattr(threading.currentThread(), 'testing', False):
+            if auto_commit is True and not getattr(threading.current_thread(), 'testing', False):
                 self._cr.commit()
 
     def resend_failed(self):
@@ -142,7 +145,7 @@ class SmsSms(models.Model):
         res = None
         try:
             # auto-commit except in testing mode
-            auto_commit = not getattr(threading.currentThread(), 'testing', False)
+            auto_commit = not getattr(threading.current_thread(), 'testing', False)
             res = self.browse(ids).send(unlink_failed=False, unlink_sent=True, auto_commit=auto_commit, raise_exception=False)
         except Exception:
             _logger.exception("Failed processing SMS queue")
