@@ -3,20 +3,32 @@
 
 from datetime import datetime
 from dateutil.relativedelta import relativedelta
+from lxml import etree
 
 from odoo import fields
 from odoo.addons.survey.tests import common
+from odoo.addons.test_mail.tests.common import MailCommon
 from odoo.exceptions import UserError
 from odoo.tests import Form
 from odoo.tests.common import users
 
 
-class TestSurveyInvite(common.TestSurveyCommon):
+class TestSurveyInvite(common.TestSurveyCommon, MailCommon):
 
     def setUp(self):
         res = super(TestSurveyInvite, self).setUp()
         # by default signup not allowed
         self.env["ir.config_parameter"].set_param('auth_signup.invitation_scope', 'b2b')
+        view = self.env.ref('survey.survey_invite_view_form').sudo()
+        tree = etree.fromstring(view.arch)
+        # Remove the invisible on `emails` to be able to test the onchange `_onchange_emails`
+        # which raises an error when attempting to change `emails`
+        # while the survey is set with `users_login_required` to True
+        # By default, `<field name="emails"/>` is invisible when `survey_users_login_required` is True,
+        # making it normally impossible to change by the user in the web client by default.
+        # For tests `test_survey_invite_authentication_nosignup` and `test_survey_invite_token_internal`
+        tree.xpath('//field[@name="emails"]')[0].attrib.pop('attrs')
+        view.arch = etree.tostring(tree)
         return res
 
     @users('survey_manager')
@@ -30,13 +42,13 @@ class TestSurveyInvite(common.TestSurveyCommon):
             # no page
             self.env['survey.survey'].create({'title': 'Test survey'}),
             # no questions
-            self.env['survey.survey'].create({'title': 'Test survey', 'question_and_page_ids': [(0, 0, {'is_page': True, 'title': 'P0', 'sequence': 1})]}),
+            self.env['survey.survey'].create({'title': 'Test survey', 'question_and_page_ids': [(0, 0, {'is_page': True, 'question_type': False, 'title': 'P0', 'sequence': 1})]}),
             # closed
             self.env['survey.survey'].with_user(self.survey_manager).create({
                 'title': 'S0',
                 'active': False,
                 'question_and_page_ids': [
-                    (0, 0, {'is_page': True, 'title': 'P0', 'sequence': 1}),
+                    (0, 0, {'is_page': True, 'question_type': False, 'title': 'P0', 'sequence': 1}),
                     (0, 0, {'title': 'Q0', 'sequence': 2, 'question_type': 'text_box'})
                 ]
             })
@@ -72,6 +84,16 @@ class TestSurveyInvite(common.TestSurveyCommon):
         self.assertEqual(answers.mapped('partner_id'), self.customer)
         self.assertEqual(set(answers.mapped('deadline')), set([deadline]))
 
+        with self.subTest('Warning when inviting an already invited partner'):
+            action = self.survey.action_send_survey()
+            invite_form = Form(self.env[action['res_model']].with_context(action['context']))
+            invite_form.partner_ids.add(self.customer)
+
+            self.assertIn(self.customer, invite_form.existing_partner_ids)
+            self.assertEqual(invite_form.existing_text,
+                             'The following customers have already received an invite: Caroline Customer.')
+
+
     @users('survey_manager')
     def test_survey_invite_authentication_nosignup(self):
         Answer = self.env['survey.user_input']
@@ -102,7 +124,7 @@ class TestSurveyInvite(common.TestSurveyCommon):
     @users('survey_manager')
     def test_survey_invite_authentication_signup(self):
         self.env["ir.config_parameter"].sudo().set_param('auth_signup.invitation_scope', 'b2c')
-        self.survey.invalidate_cache()
+        self.env.invalidate_all()
         Answer = self.env['survey.user_input']
 
         self.survey.write({'access_mode': 'public', 'users_login_required': True})
@@ -125,6 +147,21 @@ class TestSurveyInvite(common.TestSurveyCommon):
             set([self.customer.email, self.user_emp.email, self.user_portal.email]))
         self.assertEqual(answers.mapped('partner_id'), self.customer | self.user_emp.partner_id | self.user_portal.partner_id)
 
+    @users('survey_manager')
+    def test_survey_invite_email_from(self):
+        # Verifies whether changing the value of the "email_from" field reflects on the receiving end.
+        action = self.survey.action_send_survey()
+        invite_form = Form(self.env[action['res_model']].with_context(action['context']))
+        invite_form.partner_ids.add(self.survey_user.partner_id)
+        invite_form.template_id.write({'email_from':'{{ object.partner_id.email_formatted }}'})
+        invite = invite_form.save()
+        with self.mock_mail_gateway():
+            invite.action_invite()
+
+        self.assertEqual(len(self._new_mails), 1, "A new mail.mail should have been created")
+        mail = self._new_mails[0]
+        self.assertEqual(mail.email_from, self.survey_user.email_formatted)
+
     @users('survey_manager')
     def test_survey_invite_public(self):
         Answer = self.env['survey.user_input']
