@@ -5,8 +5,14 @@ from . import test_onchange
 from . import test_attributes
 from . import test_one2many
 from . import test_many2many
+from . import test_properties
+from . import test_json_field
 from . import test_qweb_float
+from . import test_indexed_translation
+from . import test_related_translation
 from . import test_ui
 from . import test_domain
 from . import test_schema
 from . import test_company_checks
+from . import test_unlink
+from . import test_views
