@@ -17,17 +17,24 @@
         'views/snippets/s_product_list.xml',
         'views/snippets/s_call_to_action.xml',
         'views/snippets/s_cover.xml',
+        'views/snippets/s_three_columns.xml',
+        'views/snippets/s_quotes_carousel.xml',
+        'views/snippets/s_text_image.xml',
+        'views/snippets/s_product_catalog.xml',
     ],
     'images': [
-        'static/description/Kiddo_description.png',
+        'static/description/kiddo_description.jpg',
         'static/description/kiddo_screenshot.jpg',
     ],
     'images_preview_theme': {
         'website.s_banner_default_image': '/theme_kiddo/static/src/img/snippets/s_banner.jpg',
-        'website.s_picture_default_image': '/theme_kiddo/static/src/img/content/content_img_15.jpg',
+        'website.s_image_text_default_image': '/theme_kiddo/static/src/img/snippets/s_image_text.jpg',
+        'website.s_three_columns_default_image_1': '/theme_kiddo/static/src/img/snippets/s_three_columns_default_image_1.jpg',
+        'website.s_three_columns_default_image_2': '/theme_kiddo/static/src/img/snippets/s_three_columns_default_image_2.jpg',
+        'website.s_three_columns_default_image_3': '/theme_kiddo/static/src/img/snippets/s_three_columns_default_image_3.jpg',
     },
     'snippet_lists': {
-        'homepage': ['s_banner', 's_image_text', 's_picture', 's_product_list', 's_call_to_action'],
+        'homepage': ['s_banner', 's_image_text', 's_three_columns', 's_product_list', 's_call_to_action'],
     },
     'license': 'LGPL-3',
     'live_test_url': 'https://theme-kiddo.odoo.com',
