@@ -1,9 +1,7 @@
 /** @odoo-module */
 
 import wTourUtils from 'website.tour_utils';
-
-const core = require("web.core");
-const _t = core._t;
+import { _t } from 'web.core';
 
 const snippets = [
     {
@@ -33,6 +31,7 @@ const snippets = [
 ];
 
 wTourUtils.registerThemeHomepageTour("notes_tour", [
+    wTourUtils.assertCssVariable('--color-palettes-name', '"notes-1"'),
     wTourUtils.dragNDrop(snippets[0]),
     wTourUtils.clickOnText(snippets[0], 'h1'),
     wTourUtils.goBackToBlocks(),
