@@ -1,6 +1,6 @@
 /** @odoo-module **/
 
-import { parseArgs } from "./py_utils";
+import { parseArgs } from "./py_parser";
 
 // -----------------------------------------------------------------------------
 // Errors
@@ -111,7 +111,7 @@ function ord2ymd(n) {
         };
     }
 
-    let leapyear = n1 === 3 && (n4 !== 24 || n100 == 3);
+    const leapyear = n1 === 3 && (n4 !== 24 || n100 == 3);
     assert(leapyear == isLeap(year));
     let month = (n + 50) >> 5;
     let preceding = DAYS_BEFORE_MONTH[month] + (month > 2 && leapyear ? 1 : 0);
@@ -176,7 +176,7 @@ function tmxxx(year, month, day, hour, minute, second, microsecond) {
     // for a datetime object, but we don't care about that here).
     // If day is out of bounds, what to do is arguable, but at least the
     // method here is principled and explainable.
-    let dim = daysInMonth(year, month);
+    const dim = daysInMonth(year, month);
     if (day < 1 || day > dim) {
         // Move day-1 days from the first of the month.  First try to
         // get off cheap if we're only one day out of range (adjustments
@@ -198,7 +198,7 @@ function tmxxx(year, month, day, hour, minute, second, microsecond) {
                 ++year;
             }
         } else {
-            let r = ord2ymd(ymd2ord(year, month, 1) + (day - 1));
+            const r = ord2ymd(ymd2ord(year, month, 1) + (day - 1));
             year = r.year;
             month = r.month;
             day = r.day;
@@ -224,10 +224,18 @@ export class PyDate {
      * @returns {PyDate}
      */
     static today() {
-        const now = new Date();
-        const year = now.getUTCFullYear();
-        const month = now.getUTCMonth() + 1;
-        const day = now.getUTCDate();
+        return this.convertDate(new Date());
+    }
+
+    /**
+     * Convert a date object into PyDate
+     * @param {Date} date
+     * @returns {PyDate}
+     */
+    static convertDate(date) {
+        const year = date.getUTCFullYear();
+        const month = date.getUTCMonth() + 1;
+        const day = date.getUTCDate();
         return new PyDate(year, month, day);
     }
 
@@ -323,13 +331,21 @@ export class PyDateTime {
      * @returns {PyDateTime}
      */
     static now() {
-        const now = new Date();
-        const year = now.getUTCFullYear();
-        const month = now.getUTCMonth() + 1;
-        const day = now.getUTCDate();
-        const hour = now.getUTCHours();
-        const minute = now.getUTCMinutes();
-        const second = now.getUTCSeconds();
+        return this.convertDate(new Date());
+    }
+
+    /**
+     * Convert a date object into PyDateTime
+     * @param {Date} date
+     * @returns {PyDateTime}
+     */
+    static convertDate(date) {
+        const year = date.getUTCFullYear();
+        const month = date.getUTCMonth() + 1;
+        const day = date.getUTCDate();
+        const hour = date.getUTCHours();
+        const minute = date.getUTCMinutes();
+        const second = date.getUTCSeconds();
         return new PyDateTime(year, month, day, hour, minute, second, 0);
     }
 
@@ -473,7 +489,7 @@ export class PyDateTime {
      * @returns {PyDateTime}
      */
     to_utc() {
-        const d = new Date(this.year, this.month, this.day, this.hour, this.minute, this.second);
+        const d = new Date(this.year, this.month -1, this.day, this.hour, this.minute, this.second);
         const timedelta = PyTimeDelta.create({ minutes: d.getTimezoneOffset() });
         return this.add(timedelta);
     }
