@@ -8,7 +8,6 @@
     'version': '1.0',
     'summary': 'Bridge module to support quiz features during "live" tracks. ',
     'website': 'https://www.odoo.com/app/events',
-    'description': "",
     'depends': [
         'website_event_track_live',
         'website_event_track_quiz',
@@ -16,16 +15,11 @@
     'data': [
         'views/event_track_templates_page.xml',
     ],
-    'demo': [
-    ],
-    'application': False,
     'installable': True,
     'auto_install': True,
     'assets': {
         'web.assets_frontend': [
             'website_event_track_live_quiz/static/src/js/**/*',
-        ],
-        'web.assets_qweb': [
             'website_event_track_live_quiz/static/src/xml/**/*',
         ],
     },
